@@ -6,7 +6,7 @@ from loguru import logger
 from typing import ClassVar
 from .base import *
 from ..flavours import _pathz_windows_flavour, _pathz_posix_flavour
-
+from ..aiopath.wrap import to_thread
 
 if TYPE_CHECKING:
     try: from ..generic import PathLike
@@ -123,6 +123,18 @@ class CloudFileSystemPath(Path, CloudFileSystemPurePath):
         if self._bucket in self.parts[1]: return self._pathlike.join(*self.parts[2:])
         return self._pathlike.join(*self.parts)
     
+    def get_path_key(self, filename: Optional[str] = None) -> str:
+        """
+        Used to return relative/path/to/file.ext
+        """
+        filename = filename or self.name
+        parts = None
+        if self._bucket in self.parts[0]: parts = self.parts[1:-1]
+        elif self._bucket in self.parts[1]: parts = self.parts[2:-1]
+        else: parts = self.parts[:-1]
+        return self._pathlike.join(*parts, filename)
+
+    
     @property
     def _cloudstr(self) -> str:
         """
@@ -1419,12 +1431,16 @@ class CloudFileSystemPath(Path, CloudFileSystemPurePath):
     def url(self, **kwargs):
         return self._accessor.url(self._cloudpath, **kwargs)
     
+
+    async def async_url(self, **kwargs):
+        return await self._accessor.async_url(self._cloudpath, **kwargs)
+
     def setxattr(self, **kwargs):
         return self._accessor.setxattr(self._cloudpath, **kwargs)
     
-    async def async_url(self, **kwargs):
-        return await self._accessor.async_url(self._cloudpath, **kwargs)
-    
+    async def async_setxattr(self, **kwargs):
+        return await self._accessor.async_setxattr(self._cloudpath, **kwargs)
+        
     def cloze(self, **kwargs):
         if self._fileio: 
             self._fileio.commit()
